// --------------------------------------------------------
#include <fstream>
#include <sstream>
#include <iostream>

// --------------------------------------------------------

using namespace std;

// --------------------------------------------------------

#include "CPersistFactory.h"
#include "IPersistObject.h"
#include "IPersistType.h"
#include "CException.h"
// --------------------------------------------------------
map<const char *,IPersistType *,CPersistFactory::CStrcmp> *CPersistFactory::s_Types=NULL;
persist_string                                             CPersistFactory::s_AppVersion;
// --------------------------------------------------------
CPersistFactory::CPersistFactory()
{
  cerr << "Persist factory created v" 
    << PERSIST_FACTORY_MAJOR 
    << "." 
    << PERSIST_FACTORY_MINOR 
#ifdef PERSIST_XML
    << " XML enabled"
#endif
    << endl << " Persist - (c) Sylvain Lefebvre 2002 - http://www.aracknea.net" << endl << endl;
  //  cout << "Sylvain Lefebvre 2002 - Sylvain.Lefebvre@imag.fr" << endl;
  m_bLoadInProgress=false;
  m_bBeforeFirstLoad=true;
# ifdef PERSIST_DEBUG
  if (s_Types != NULL)
  {
    cerr << "  Registred types: " << endl;
    for (map<const char *,IPersistType *,CPersistFactory::CStrcmp>::iterator I=s_Types->begin();
      I!=s_Types->end();I++)
      cerr << '\t' << (*I).first << endl;
    cerr << endl;
  }
  else
    cerr << "  No registred types !" << endl;
# endif
  m_szCurrentFileName[0]='\0';
  m_iIdsOffset=0;
  m_bMerge=false;
}
// --------------------------------------------------------
CPersistFactory::~CPersistFactory()
{
  delete (s_Types);
  cout << "CPersistFactory destroyed." << cout;
}

// --------------------------------------------------------
void CPersistFactory::setApplicationVersionString(const string& s)

{

# ifdef PERSIST_DEBUG

  cerr << "PERSIST - Application Version String: " << s << endl;

# endif
  s_AppVersion=s;

}

// --------------------------------------------------------
void CPersistFactory::childFromId(int id,CPtrStock *ptr)
{
  if (id < 0)
  {
#   ifdef PERSIST_DEBUG
    cerr << "child = (null)" << endl; 
#   endif
    ptr->set(NULL);
    return;
  }
# ifdef PERSIST_DEBUG
  cerr << "child id = " << id << endl; 
# endif
  if (m_Registred[id] != NULL)
  {
#   ifdef PERSIST_DEBUG
    cerr << " found !" << endl;
#   endif
    ptr->set(m_Registred[id]);
  }
  else
  {
#   ifdef PERSIST_DEBUG
    cerr << " to be continued ..." << endl;
#   endif
    m_Needed[id].push_back(ptr);
    ptr->set(NULL);
  }
}
// --------------------------------------------------------
void CPersistFactory::rawLoadChild(istream& s,CPtrStock *ptr)
{
  int  id;
  char type;

  if (s_Types == NULL)
    throw CException("CPersistFactory: cannot load anything: no registred types ! Use PERSIST_ID macro to register your persitent objects.");
  s >> id;
  if (id >= 0)
  {
    s >> type;
    if (type == 'O')
      childFromId(id,ptr);
    else if (type == 'D')
      ptr->set(m_RegistredData[id]);
    else
      throw CException("PERSIST: rawLoadChild - unkown object type ('%c') !",type);
  }
  else
    childFromId(id,ptr);
}
// --------------------------------------------------------
void CPersistFactory::rawSaveChild(ostream& s,IPersistObject *o)
{
  if (o == NULL)
    s << -1 << ' ';
  else
  {
    if (!o->idDefined())
    {
      if (o->isDynamic())
        registerObject(o);
      else
        throw CException("PERSIST - cannot automatically register a non dynamic object !\n(if you are saving an instance, please be sure that all instances concerned by the save can be safely created by persist).");
    }
    s << (o->getId()+m_iIdsOffset) << ' ';
    if (o->isDynamic())
      s << 'O' << ' '; // object
    else
      s << 'D' << ' ';  // data
  }
}
// --------------------------------------------------------
void CPersistFactory::registerObject(IPersistObject *o)
{
  if (o->idDefined())
    return;
  if (!m_bLoadInProgress)
  {
    if (m_bBeforeFirstLoad)
    {
#     ifdef PERSIST_DEBUG
      cerr << "CPersistFactory: registring object before first load -> redirecting to data" << endl;
#     endif
      registerData(o);
      o->forceDynamic(false);
    }
    else
    {
      m_Registred.push_back(o);
      o->setId(m_Registred.size()-1);
#     ifdef PERSIST_DEBUG
      cerr << "CPersistFactory: registring object " << o->getId() << endl;
#     endif
    }
  }
}
// --------------------------------------------------------
void CPersistFactory::unregisterObject(IPersistObject *o)
{
  // the object is removed and all ids updated
  if (o->idDefined())
  {
#   ifdef PERSIST_DEBUG
    cerr << "CPersistFactory: Unregistering object (id = " << o->getId() << ")" << endl;
#   endif
    m_Registred[o->getId()]=NULL;
    o->setId(-1);
    compactIds();
#   ifdef PERSIST_DEBUG
    {
      map<const char *,int,CStrcmp> count;

      cerr << "CPersistFactory: " << m_Registred.size() << " persistent objects remains:" << endl;
      for (vector<IPersistObject *>::iterator Id=m_Registred.begin();Id!=m_Registred.end();Id++)
        count[(*Id)->getPersistType()->m_szTypeId]++;
      for (map<const char *,int,CStrcmp>::iterator Im=count.begin();Im!=count.end();Im++)
        cout << '\t' << (*Im).first << " (" << (*Im).second << ")" << endl;
    }
#   endif
  }
}
// --------------------------------------------------------
void CPersistFactory::registerType(IPersistType *t)
{
  static bool first=true;

  if (first)
  {
    s_Types=new map<const char *,IPersistType *,CPersistFactory::CStrcmp>;
    first=false;
  }
  (*s_Types)[t->m_szTypeId]=t;
}
// --------------------------------------------------------
void CPersistFactory::registerData(IPersistObject *d)
{
  m_RegistredData.push_back(d);
  d->setId(m_RegistredData.size()-1);
# ifdef PERSIST_DEBUG
  cerr << "CPersistFactory: registring data " << d->getId() << endl;
# endif
}
// --------------------------------------------------------
void CPersistFactory::unregisterData(IPersistObject *d)
{
  // TODO : is this authorized ?
  if (d->idDefined())
  {
#   ifdef PERSIST_DEBUG
    cerr << "CPersistFactory: unregistring data " << d->getId() << endl;
#   endif
    m_RegistredData[d->getId()]=NULL;
    d->setId(-1);
  }
}
// --------------------------------------------------------
void CPersistFactory::unregister(IPersistObject *o)
{
  if (!o->idDefined())
    return;
  if (o->isDynamic())
    unregisterObject(o);
  else
    unregisterData(o);
}
// --------------------------------------------------------
void CPersistFactory::rawSaveObject(ostream& s,IPersistObject *o)
{
  const char *type=NULL;

  if (o->getPersistType() == NULL)
    throw CException("CPersistFactory: Object has no type id !!\n\tPlease use PERSIST_ID macro to create a type id for your objects.");
  s << (o->getId()+m_iIdsOffset) << ',';
  type=o->getPersistType()->m_szTypeId;
  if (type == NULL)
    throw CException("CPersistFactory::saveObject - unknown type !");
  s << type << ',';
  o->save(s);
  s << ' ';
  o->rawPersistLoadSaveMembers(NULL,&s);
}
// --------------------------------------------------------
IPersistObject *CPersistFactory::rawBuildObject(istream& s)
{
# define BUF_SIZE 256
  char               c='\0';
  int                id;
  IPersistObject    *o;
  IPersistType      *t;
  char               buf[BUF_SIZE];
  int                ib=0;

  // read id
  s >> id >> c;
  if (s.eof())
    return (NULL);
  m_iNbObjectsLoaded++;
  if (c != ',')
  {
    s.clear(ios::badbit);
    throw CException("CPersistFactory: syntax error ! (corrupted id - %d objects loaded)",m_iNbObjectsLoaded);
  }
  // read type
  while (1) 
  {
    if (s.get(c))
    {
      if (c == ',')
        break;
      buf[ib++]=c;
      if (ib >= BUF_SIZE-1)
        throw CException("CPersistFactory: type name is too long (%d max)",BUF_SIZE);
    }
    else
    {
      s.clear(ios::badbit);
      throw CException("CPersistFactory: syntax error ! (corrupted type - %d objects loaded)",m_iNbObjectsLoaded);
    }
  }
  buf[ib]='\0';
  // construct from type
  t=(*s_Types)[buf];
  if (t == NULL)
    throw CException("CPersistFactory: type %s not found in registred types !",buf);
  o=(*t)(s);
  o->rawPersistLoadSaveMembers(&s,NULL);
  // give it's id to the object
  id+=m_iIdsOffset;
  o->setId(id);
  m_Registred[id]=o;
# ifdef PERSIST_DEBUG
  cerr << "building (type " << buf << " id = " << id << " object adr = " << (int)o << ")" << endl;
# endif
  // if the object is needed by another, update ptrs !
  for (list<CPtrStock *>::iterator I=m_Needed[id].begin();
    I!=m_Needed[id].end();I++)
  {
#   ifdef PERSIST_DEBUG
    cerr << "updating 1 ref " 
      << "(ref on " << o->getId() << " - " << o->getPersistType()->m_szTypeId << ")"
      << endl;
#   endif
    (*I)->set(o);
  }
  // done
  return (o);
}
// --------------------------------------------------------
void CPersistFactory::rawSave(const char *n)
{
  // open stream
  ofstream out(n);

  if (!out)
    throw CException("CPersistFactory: file %s cannot be written !",n);
  strncpy(m_szCurrentFileName,n,PERSIST_FILE_NAME_MAX_SIZE);
  // save persistent objects
  rawSaveObjects(out);
  // save persistent datas
  rawSaveDatas(out);
  // close stream
  out.close();
  m_szCurrentFileName[0]='\0';
}
// --------------------------------------------------------
void CPersistFactory::rawLoad(const char *n)
{
  // open stream
  ifstream          in(n);

  // first load
  m_bBeforeFirstLoad=false;
  // if no file
  if (!in)
    throw CException("CPersistFactory: file %s cannot be read !",n);
  // state
  m_bLoadInProgress=true;
  // load persistent objects
  rawLoadObjects(in);
  // load persistent datas
  rawLoadDatas(in);
  // load done
  m_bLoadInProgress=false;
  // cloas stream
  in.close();
}
// --------------------------------------------------------
void CPersistFactory::rawSaveObjects(ostream& out)
{
  // write number of objects
  out << m_Registred.size() << endl;
  // save persistent objects
  for (vector<IPersistObject *>::iterator I=m_Registred.begin();
    I!=m_Registred.end();I++)
  {
    if ((*I) == NULL)
      continue;
    rawSaveObject(out,(*I));
    out << endl;
  }
}
// --------------------------------------------------------
void CPersistFactory::rawLoadObjects(istream& in)
{
  IPersistObject *o;
  int             nb,i;

  m_iNbObjectsLoaded=0;
  // clear all
  m_Registred.clear();
  for (vector<list<CPtrStock *> >::iterator I1=m_Needed.begin();I1!=m_Needed.end();I1++)
    for (list<CPtrStock *>::iterator J1=(*I1).begin();J1!=(*I1).end();J1++)
      delete (*J1);
  m_Needed.clear();
  // read number of objects
  in >> nb;
  m_Needed.resize(nb);
  m_Registred.resize(nb);
  for (vector<IPersistObject *>::iterator I2=m_Registred.begin();
    I2!=m_Registred.end();I2++)
    (*I2)=NULL;
  // build objects
  for (i=0;i<nb;i++)
  { 
    o=rawBuildObject(in);
    if (o == NULL)
      throw CException("CPersistFactory::load - error while building objects");
  }
  cout << endl;
  // compact ids
  compactIds();
  // finalize objects
  for (vector<IPersistObject *>::iterator I3=m_Registred.begin();
    I3!=m_Registred.end();I3++)
    if ((*I3) != NULL)
      (*I3)->finalize();
    else
      throw CException("CPersistFactory::load - coherence fault");
  // clear needed
  for (vector<list<CPtrStock *> >::iterator I4=m_Needed.begin();I4!=m_Needed.end();I4++)
    for (list<CPtrStock *>::iterator J2=(*I4).begin();J2!=(*I4).end();J2++)
      delete (*J2);
  m_Needed.clear();
}
// --------------------------------------------------------
bool CPersistFactory::rawSaveInstance(IPersistObject *o,const char *n)
{
  // open string stream
  ostringstream out;
  int           start=m_Registred.size();

#ifdef PERSIST_DEBUG
  cerr << "CPersistFactory::rawSaveInstance \"" << n << "\"" << endl;
#endif
  if (!out)
    throw CException("PERSIST: file %s cannot be written !",n);
  strncpy(m_szCurrentFileName,n,PERSIST_FILE_NAME_MAX_SIZE);
  // push all ids
  for (vector<IPersistObject *>::iterator I1=m_Registred.begin();
    I1!=m_Registred.end();I1++)
    (*I1)->pushId();
  // save all concerned instances
  registerObject(o);
  m_iIdsOffset=-(o->getId());
  for (int i=start;i<(int)m_Registred.size();i++)
  {
    if (m_Registred[i] == NULL)
      continue;
    rawSaveObject(out,m_Registred[i]); // will automatically register child objects with rawSaveChild !
    out << endl;
  }
  // need to add the number of saved instances at the begining of the file ...
  ofstream      file(n);
  file << (m_Registred.size()-start) << endl;
  file << out.str().c_str();
  file.close();
  // forget all newly registred objects
  m_Registred.resize(start);
  // pop all ids
  for (vector<IPersistObject *>::iterator I2=m_Registred.begin();
    I2!=m_Registred.end();I2++)
    (*I2)->popId();
  m_szCurrentFileName[0]='\0';
  m_iIdsOffset=0;
  return (true);
}
// --------------------------------------------------------
IPersistObject *CPersistFactory::rawLoadInstance(const char *n)
{
  int i,first=0;

#ifdef PERSIST_DEBUG
  cerr << "CPersistFactory::rawLoadInstance \"" << n << "\"" << endl;
#endif
  // open stream
  ifstream in(n);
  if (!in)
    throw CException("PERSIST: unable to open %s !",n);
  // state
  m_bLoadInProgress=true;
  // --------------
  // init load objects
  // --------------
  m_iNbObjectsLoaded=0;
  first=m_Registred.size();
  // clear needed list    
  for (vector<list<CPtrStock *> >::iterator I1=m_Needed.begin();I1!=m_Needed.end();I1++)
  {
    for (list<CPtrStock *>::iterator J1=(*I1).begin();J1!=(*I1).end();J1++)
      delete (*J1);
    (*I1).clear();
  }
  m_Needed.clear();
  // --------------
  // load objects
  // --------------
  int nb;
  // read number of objects
  in >> nb;
  m_Needed.resize(m_Registred.size()+nb);
  m_Registred.resize(m_Registred.size()+nb);
  for (i=first;i<(int)m_Registred.size();i++)
    m_Registred[i]=NULL;
  // build objects
  for (i=0;i<nb;i++)
  { 
    IPersistObject* o=rawBuildObject(in);
    if (o == NULL)
      throw CException("PERSIST: rawLoadInstance - error while building objects");
  }
  // close stream
  in.close();
  // --------------
  // finalize objects
  // --------------
  // finalize objects
  for (i=first;i<(int)m_Registred.size();i++)
  {
    if (m_Registred[i] != NULL)
      m_Registred[i]->finalize();
    else
      throw CException("CPersistFactory::load - coherence fault");
  }
  // clear needed
  for (vector<list<CPtrStock *> >::iterator I2=m_Needed.begin();I2!=m_Needed.end();I2++)
  {
    for (list<CPtrStock *>::iterator J2=(*I2).begin();J2!=(*I2).end();J2++)
      delete (*J2);
    (*I2).clear();
  }
  m_Needed.clear();  
  // load done
  m_bLoadInProgress=false;
  // update loaded table
  m_Loaded.clear();
  m_Loaded.push_back(m_Registred[first]);
  // return instance
  return (m_Registred[first]);
}
// --------------------------------------------------------
void CPersistFactory::compactIds()
{
  int                                id=0;
  vector<IPersistObject *>::iterator I;

  I=m_Registred.begin();
  while (I!=m_Registred.end())
  {
    if ((*I) == NULL)
      I=m_Registred.erase(I);
    else
    {
      (*I)->setId(id++);
      I++;
    }
  }
}
// --------------------------------------------------------
void CPersistFactory::rawSaveDatas(ostream& out)
{
  out << m_RegistredData.size() << endl;
  for (vector<IPersistObject *>::iterator I=m_RegistredData.begin();
    I!=m_RegistredData.end();I++)
  {
    if ((*I) == NULL)
      continue;
# ifdef PERSIST_DEBUG
    cerr << "saving data for id = " << ((*I)->getId()+m_iIdsOffset) << " offset = " << m_iIdsOffset << endl;
# endif
    out << ((*I)->getId()+m_iIdsOffset) << ' ';
    (*I)->save(out);
    (*I)->rawPersistLoadSaveMembers(NULL,&out);
    out << endl;
  }
}
// --------------------------------------------------------
void CPersistFactory::rawLoadDatas(istream& in)
{
  int i,nb,id;

  in >> nb;
  if (in.eof() || nb == -1)
  {
    cerr << "CPersistFactory::loadDatas - No data in file, setting to defaults." << endl;
    defaultDatas();
    return;
  }
  if (nb != m_RegistredData.size())
  {
    cerr << ("CPersistFactory::loadDatas - Application should have the same persistent *data* objects loaded in the same order when load and save are called.") << endl << " -> setting to defaults." << endl;
    defaultDatas();
    return;
  }
  for (i=0;i<nb;i++)
  {
    in >> id;
# ifdef PERSIST_DEBUG
    cerr << "reading data for id = " << id << endl;
# endif
    if (id < 0 || id >= (int)m_RegistredData.size())
      throw CException("CPersistFactory::loadDatas - corrupted id (%d)",id);
    if (m_RegistredData[id] != NULL)
    {
      m_RegistredData[id]->load(in);
      m_RegistredData[id]->rawPersistLoadSaveMembers(&in,NULL);
    }
    else
      throw CException("CPersistFactory::loadDatas - IPersistObject (%d) was unregistred before loading !",id);
  }
  for (vector<IPersistObject *>::iterator I=m_RegistredData.begin();
    I!=m_RegistredData.end();I++)
  {
    if ((*I) != NULL)
      (*I)->finalize();
  }
}
// --------------------------------------------------------
void CPersistFactory::defaultDatas()
{
  for (vector<IPersistObject *>::iterator I=m_RegistredData.begin();
    I!=m_RegistredData.end();I++)
  {
    if ((*I) != NULL)
      (*I)->failed();
  }
}
// --------------------------------------------------------
//
//                       XML Persist
//
// --------------------------------------------------------
#ifdef PERSIST_XML
// --------------------------------------------------------
void CPersistFactory::xmlSaveChild(ostream& s,
                                   IPersistObject *o)
{
  if (o == NULL)
    s << " id=\"-1\" mem=\"object\"";
  else
  {
    if (!o->idDefined())
    {
      if (o->isDynamic())
        registerObject(o);
      else
        throw CException("PERSIST - cannot automatically register a non dynamic object !\n(if you are saving an instance, please be sure that all instances concerned by the save can be safely created by persist).");
    }
    s << " id=\"" << (o->getId()+m_iIdsOffset) << "\"";
    if (o->isDynamic())
      s << " mem=\"object\"";
    else
      s << " mem=\"data\"";      
  }
}
// --------------------------------------------------------
void CPersistFactory::xmlSaveObject(ostream& s,IPersistObject *o)
{
  const char *type=NULL;

  if (o->getPersistType() == NULL)
    throw CException("CPersistFactory: Object has no type id !!\n\tPlease use PERSIST_ID macro to create a type id for your objects.");
  type=o->getPersistType()->m_szTypeId;
  if (type == NULL)
    throw CException("CPersistFactory::saveObject - unknown type !");
  s << "  <instance type=\"" << type 
    << "\" id=\"" << (o->getId()+m_iIdsOffset)
    << "\">" << endl;
  // object members
  vector<string> empty;
  o->xmlPersistLoadSaveMembers("",empty,&s);
  // custom
  s << "    <custom>";
  o->save(s);
  s << "</custom>" << endl;
  // done
  s << "  </instance>" << endl;
}
// --------------------------------------------------------
void CPersistFactory::xmlSaveObjects(ostream& out)
{
  // write number of objects
  out << "<persist_objects size=\"" << m_Registred.size() 
    << "\">" << endl;
  // save persistent objects
  for (vector<IPersistObject *>::iterator I=m_Registred.begin();
    I!=m_Registred.end();I++)
  {
    if ((*I) == NULL)
      continue;
    xmlSaveObject(out,(*I));
  }
  out << "</persist_objects>" << endl;
}
// --------------------------------------------------------
void CPersistFactory::xmlSaveDatas(ostream& out)
{
  vector<string> empty;

  out << "<persist_datas size=\"" << m_RegistredData.size() 
    << "\">" << endl;
  for (vector<IPersistObject *>::iterator I=m_RegistredData.begin();
    I!=m_RegistredData.end();I++)
  {
    if ((*I) == NULL)
      continue;
# ifdef PERSIST_DEBUG
    cerr << "saving xml data for id = " << (*I)->getId() << " offset = " << m_iIdsOffset << endl;
# endif
    out << "  <data type=\"" << (*I)->getPersistType()->m_szTypeId << "\" id=\"" 
      << ((*I)->getId()+m_iIdsOffset) << "\">" << endl;

    // members
    (*I)->xmlPersistLoadSaveMembers("",empty,&out);
    // custom

    out << "    <custom>";
    (*I)->save(out);
    out << "</custom>" << endl;
    out << "  </data>" << endl;
  }
  out << "</persist_datas>" << endl;
}
// --------------------------------------------------------
void CPersistFactory::xmlSave(const char *n)
{
  // open stream
  ofstream out(n);

  if (m_bLoadInProgress)
    throw CException("PERSIST: fatal error - last load was not completed !",n);
  if (!out)
    throw CException("CPersistFactory: file %s cannot be written !",n);
  m_iIdsOffset=0;
  // loading xml file
  strncpy(m_szCurrentFileName,n,PERSIST_FILE_NAME_MAX_SIZE);
  out << "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" << endl;
  out << "<!DOCTYPE persist SYSTEM \"persist.dtd\">" << endl;
  out << "<persist version=\"" << PERSIST_FACTORY_MAJOR 
    << '.' << PERSIST_FACTORY_MINOR 
    << "\">" << endl;
  // save application data
  out << "<application>" << endl;
  out << "<version>" << s_AppVersion << "</version>" << endl;
  out << "<comment>" << endl;
  out << "</comment>" << endl;
  if (s_Types != NULL)
  {
    for (map<const char *,IPersistType *,CPersistFactory::CStrcmp>::iterator I=s_Types->begin();
      I!=s_Types->end();I++)
      out << "<type>" << (*I).first << "</type>" << endl;
  }
  out << "</application>" << endl;
  // save persistent objects
  xmlSaveObjects(out);
  // save persistent datas
  xmlSaveDatas(out);
  // close stream
  out << "</persist>" << endl;
  out.close();
  m_szCurrentFileName[0]='\0';
}
// --------------------------------------------------------
bool CPersistFactory::xmlSaveInstance(IPersistObject *o,const char *n)
{
  // open stream
  ofstream out(n);
  int      start=m_Registred.size();

#ifdef PERSIST_DEBUG
  cerr << "CPersistFactory::xmlSaveInstance \"" << n << "\"" << endl;
#endif
  if (!out)
    throw CException("CPersistFactory: file %s cannot be written !",n);
  strncpy(m_szCurrentFileName,n,PERSIST_FILE_NAME_MAX_SIZE);
  out << "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" << endl;
  out << "<!DOCTYPE persist SYSTEM \"persist.dtd\">" << endl;
  out << "<persist version=\"" << PERSIST_FACTORY_MAJOR 
    << '.' << PERSIST_FACTORY_MINOR 
    << "\">" << endl;
  // save application data
  out << "<application>" << endl;
  out << "<version>" << s_AppVersion << "</version>" << endl;
  out << "<comment>" << endl;
  out << "</comment>" << endl;
  if (s_Types != NULL)
  {
    for (map<const char *,IPersistType *,CPersistFactory::CStrcmp>::iterator I=s_Types->begin();
      I!=s_Types->end();I++)
      out << "<type>" << (*I).first << "</type>" << endl;
  }
  out << "</application>" << endl;
  // push all ids
  for (vector<IPersistObject *>::iterator I1=m_Registred.begin();
    I1!=m_Registred.end();I1++)
    (*I1)->pushId();
  // save all concerned instances
  registerObject(o);
  m_iIdsOffset=-(o->getId());
  out << "<persist_instances>" << endl;
  for (int i=start;i<(int)m_Registred.size();i++)
  {
    if (m_Registred[i] == NULL)
      continue;
    xmlSaveObject(out,m_Registred[i]); // will automatically register child objects with xmlSaveChild !
  }
  // forget all newly registred objects
  m_Registred.resize(start);
  // pop all ids
  for (vector<IPersistObject *>::iterator I2=m_Registred.begin();
    I2!=m_Registred.end();I2++)
    (*I2)->popId();
  // close stream
  out << "</persist_instances>" << endl;
  out << "</persist>" << endl;
  out.close();
  m_szCurrentFileName[0]='\0';
  m_iIdsOffset=0;
  return (true);
}
// --------------------------------------------------------
void CPersistFactory::xmlLoad(const char *n)
{
  int i,first=0;

  // first load
  m_bBeforeFirstLoad=false;
  // --------------
  // load and parse xml tree
  // --------------
  xmlDocPtr  doc;
  xmlNodePtr root;

  doc = xmlParseFile(n);
  if (doc == NULL) 
    throw CException("Persist: unable to open %s !",n);
  root = xmlDocGetRootElement(doc);
  // validation
  /*#ifndef WIN32
  xmlValidCtxt cvp;



  cvp.userData = stderr;
  cvp.error = (xmlValidityErrorFunc) fprintf;
  cvp.warning = (xmlValidityWarningFunc) fprintf;  
  if (!xmlValidateDocument(&cvp, doc))

  throw CException("Persist: unable to load - XML document not valid !");

  #endif

  */  // state
  m_bLoadInProgress=true;

  // --------------
  // init load objects
  // --------------
  m_iNbObjectsLoaded=0;
  if (!m_bMerge)
  {
    // clear all
    m_Registred.clear();
    m_iIdsOffset=0;
  }
  else
  {
    first=m_Registred.size();
    m_iIdsOffset=first;
  }
  // clear needed list    
  for (vector<list<CPtrStock *> >::iterator I1=m_Needed.begin();I1!=m_Needed.end();I1++)
  {
    for (list<CPtrStock *>::iterator J1=(*I1).begin();J1!=(*I1).end();J1++)
      delete (*J1);
    (*I1).clear();
  }
  m_Needed.clear();
  // --------------
  // load loop
  // --------------
  xmlLoadLoop(root->children);
  // --------------
  // finalize objects
  // --------------
  // finalize objects
  for (i=first;i<(int)m_Registred.size();i++)
  {
    if (m_Registred[i] != NULL)
      m_Registred[i]->finalize();
    else
      throw CException("CPersistFactory::load - coherence fault");
  }
  // clear needed
  for (vector<list<CPtrStock *> >::iterator I2=m_Needed.begin();I2!=m_Needed.end();I2++)
  {
    for (list<CPtrStock *>::iterator J2=(*I2).begin();J2!=(*I2).end();J2++)
      delete (*J2);
    (*I2).clear();
  }
  m_Needed.clear();  
  // --------------
  // finalize datas (if datas in merge they replace older)
  // --------------
  for (vector<IPersistObject *>::iterator I3=m_RegistredData.begin();
    I3!=m_RegistredData.end();I3++)
  {
    if ((*I3) != NULL)
      (*I3)->finalize();
  }
  // --------------
  // update loaded
  // --------------
  m_Loaded.clear();
  for (i=first;i<(int)m_Registred.size();i++)
    m_Loaded.push_back(m_Registred[i]);
  // compact ids
  compactIds();
  // load done
  m_bLoadInProgress=false;
}
// --------------------------------------------------------
IPersistObject *CPersistFactory::xmlLoadInstance(const char *n)
{
  int i,first=0;

#ifdef PERSIST_DEBUG
  cerr << "CPersistFactory::xmlLoadInstance \"" << n << "\"" << endl;
#endif
  // --------------
  // load and parse xml tree
  // --------------
  xmlDocPtr  doc;
  xmlNodePtr root;

  doc = xmlParseFile(n);
  if (doc == NULL) 
    throw CException("PERSIST: unable to open %s !",n);
  root = xmlDocGetRootElement(doc);
#ifndef WIN32
  // validation
  xmlValidCtxt cvp;

  cvp.userData = stderr;
  cvp.error = (xmlValidityErrorFunc) fprintf;
  cvp.warning = (xmlValidityWarningFunc) fprintf;  
  if (!xmlValidateDocument(&cvp, doc))
    throw CException("Persist: unable to load - XML document not valid !");
#endif
  // state
  m_bLoadInProgress=true;
  // --------------
  // init load objects
  // --------------
  m_iNbObjectsLoaded=0;
  first=m_Registred.size();
  // clear needed list    
  for (vector<list<CPtrStock *> >::iterator I1=m_Needed.begin();I1!=m_Needed.end();I1++)
  {
    for (list<CPtrStock *>::iterator J1=(*I1).begin();J1!=(*I1).end();J1++)
      delete (*J1);
    (*I1).clear();
  }
  m_Needed.clear();
  // --------------
  // load loop
  // --------------
  xmlLoadLoop(root->children);
  // --------------
  // finalize objects
  // --------------
  // finalize objects
  for (i=first;i<(int)m_Registred.size();i++)
  {
    if (m_Registred[i] != NULL)
      m_Registred[i]->finalize();
    else
      throw CException("CPersistFactory::load - coherence fault");
  }
  // clear needed
  for (vector<list<CPtrStock *> >::iterator I2=m_Needed.begin();I2!=m_Needed.end();I2++)
  {
    for (list<CPtrStock *>::iterator J2=(*I2).begin();J2!=(*I2).end();J2++)
      delete (*J2);
    (*I2).clear();
  }
  m_Needed.clear();  
  // load done
  m_bLoadInProgress=false;
  // update loaded table
  m_Loaded.clear();
  m_Loaded.push_back(m_Registred[first]);
  // return instance
  return (m_Registred[first]);
}
// --------------------------------------------------------
void CPersistFactory::xmlLoadLoop(xmlNodePtr node,
                                  IPersistObject *instance)
{
  while (node != NULL)
  {
    // == Persist ==
    if (!xmlStrcmp((const xmlChar *)"persist",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << "Document was generated with XML Persist Version " 
        << xmlGetProp(node,(const xmlChar *)"version") << endl;
#endif
    }
    // == Application ==
    else if (!xmlStrcmp((const xmlChar *)"application",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << "Document contains application data" << endl;
#endif
      xmlLoadLoop(node->children);
    }
    // == Application - type ==
    else if (!xmlStrcmp((const xmlChar *)"type",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << "   => type " << xmlNodeGetContent(node) << endl;
#endif
    }
    // == Application - version ==
    else if (!xmlStrcmp((const xmlChar *)"version",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << " Version: " << xmlNodeGetContent(node) << endl;
#endif
    }
    // == Application - comment ==
    else if (!xmlStrcmp((const xmlChar *)"comment",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << " Comment: " << xmlNodeGetContent(node) << endl;
#endif
    }
    // == Persist objects ==
    else if (!xmlStrcmp((const xmlChar *)"persist_objects",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << "Objects ,";
      cerr << "  size = " << xmlGetProp(node,(const xmlChar *)"size") 
        << (m_bMerge ? " (merge)" : "")
        << endl;
#endif
      int nb=atoi((char *)xmlGetProp(node,(const xmlChar *)"size"));
      if (m_bMerge)
      {
        m_Needed.resize(m_Registred.size()+nb);
        m_Registred.resize(m_Registred.size()+nb);
        for (int i=m_iIdsOffset;i<(int)m_Registred.size();i++)
          m_Registred[i]=NULL;
      }
      else
      {
        m_Needed.resize(nb);
        m_Registred.resize(nb);
        for (vector<IPersistObject *>::iterator I=m_Registred.begin();
          I!=m_Registred.end();I++)
          (*I)=NULL;
      }
      xmlLoadLoop(node->children);
    }
    // == Persist datas ==
    else if (!xmlStrcmp((const xmlChar *)"persist_datas",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << "Datas ,";
      cerr << "  size = " << xmlGetProp(node,(const xmlChar *)"size") << endl;
#endif
      xmlLoadLoop(node->children);
    }
    // == Persist instances ==
    else if (!xmlStrcmp((const xmlChar *)"persist_instances",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << "Instances, ";
#endif
      int first=m_Registred.size();
      m_iIdsOffset=first;
      // count children
      int nb=0;
      for (xmlNodePtr p=node->children;NULL != p;p=p->next)
        if (!xmlStrcmp((const xmlChar *)"instance",p->name))
          nb++;
#ifdef PERSIST_DEBUG
      cerr << " nb = " << nb << endl;
#endif
      m_Needed.resize(m_Registred.size()+nb);
      m_Registred.resize(m_Registred.size()+nb);
      for (int i=first;i<(int)m_Registred.size();i++)
        m_Registred[i]=NULL;
      // recurs load
      bool b=m_bMerge;
      setMerge(true);
      xmlLoadLoop(node->children);
      setMerge(b);
      // no more offset
      m_iIdsOffset=0;
    }
    // == Instance ==
    else if (!xmlStrcmp((const xmlChar *)"instance",node->name))
    {
      IPersistObject   *o;
      IPersistType     *t;

#ifdef PERSIST_DEBUG
      cerr << "  Instance id=" 
        << xmlGetProp(node,(const xmlChar *)"id") 
        << " type=" 
        << xmlGetProp(node,(const xmlChar *)"type") << endl;
#endif
      // get id
      int id=atoi((char *)(xmlGetProp(node,(const xmlChar *)"id")))+m_iIdsOffset;
      // construct from type
      t=(*s_Types)[(char *)(xmlGetProp(node,(const xmlChar *)"type"))];
      if (t == NULL)
        throw CException("CPersistFactory: type %s not found in registred types !",
        (char *)xmlGetProp(node,(const xmlChar *)"type"));
      o=(*t)();
      // give it's id to the object
      o->setId(id);
      // register it
      m_Registred[id]=o;
      // init object
      o->init();
      // debug info
#     ifdef PERSIST_DEBUG
      cerr << "building (type " << (char *)(xmlGetProp(node,(const xmlChar *)"type"))
        << " id = " << id << " offset = " << m_iIdsOffset
        << " object adr = " << (int)o << ")" << endl;
#     endif
      // if the object is needed by another, update ptrs !
      for (list<CPtrStock *>::iterator I=m_Needed[id].begin();
        I!=m_Needed[id].end();I++)
      {
#       ifdef PERSIST_DEBUG
        cerr << "updating 1 ref " 
          << "(ref on " << o->getId() << " - " << o->getPersistType()->m_szTypeId << ")"
          << endl;
#       endif
        (*I)->set(o);      
      }
      // recurs
      xmlLoadLoop(node->children,o);
    }
    // == Data ==
    else if (!xmlStrcmp((const xmlChar *)"data",node->name))
    {
#     ifdef PERSIST_DEBUG
      cerr << "  Data id=" 
        << xmlGetProp(node,(const xmlChar *)"id") 
        << " type=" 
        << xmlGetProp(node,(const xmlChar *)"type") << endl;
#     endif
      // get id
      int id=atoi((char *)(xmlGetProp(node,(const xmlChar *)"id")));
      if (id < 0 || id >= (int)m_RegistredData.size())
        throw CException("CPersistFactory::xmlLoadLoop - corrupted data id (%d)",id);
      if (m_RegistredData[id] != NULL)
      {
        xmlLoadLoop(node->children,m_RegistredData[id]);
      }
      else
        throw CException("CPersistFactory::xmlLoadLoop - IPersistObject (%d) was unregistred before loading !",id);
    }
    // == Member ==
    else if (!xmlStrcmp((const xmlChar *)"member",node->name))
    {
      vector<string> strs;

#ifdef PERSIST_DEBUG
      cerr << "    Member name=" 
        << xmlGetProp(node,(const xmlChar *)"name") 
        << " value=" << xmlNodeGetContent(node) << endl;
#endif      
      strs.push_back((char *)xmlNodeGetContent(node));
      instance->xmlPersistLoadSaveMembers((char *)xmlGetProp(node,(const xmlChar *)"name"),
        strs,
        NULL);
    }
    // == Ptr ==
    else if (!xmlStrcmp((const xmlChar *)"ptr",node->name))
    {
      vector<string> strs;
      int            id;

#ifdef PERSIST_DEBUG
      cerr << "    Ptr name=" 
        << xmlGetProp(node,(const xmlChar *)"name") 
        << " id=" << xmlGetProp(node,(const xmlChar *)"id") 
        << " offset = " << m_iIdsOffset << endl;
#endif
      id=atoi((char *)xmlGetProp(node,(const xmlChar *)"id"));
      list<CPersistFactory::CPtrStock *> ptrs=
        instance->xmlPersistLoadSaveMembers((char *)xmlGetProp(node,(const xmlChar *)"name"),
        strs,
        NULL);
      if (ptrs.empty())
        throw CException("PERSIST::xmlLoadLoop - ptr %s undefined !",
        (char *)xmlGetProp(node,(const xmlChar *)"name"));
      if (ptrs.size() > 1)
        throw CException("PERSIST::xmlLoadLoop - multiple definition of ptr %s !",
        (char *)xmlGetProp(node,(const xmlChar *)"name"));
      if (!xmlStrcmp((const xmlChar *)"object",xmlGetProp(node,(const xmlChar *)"mem")))
      {
        if (id >= 0)
          id+=m_iIdsOffset;
        childFromId(id,(*ptrs.begin()));
      }
      else
        (*ptrs.begin())->set(m_RegistredData[id]);
    }
    // == Sequence ==
    else if (!xmlStrcmp((const xmlChar *)"sequence",node->name))
    {
      vector<string> strs;

#ifdef PERSIST_DEBUG
      cerr << "    Sequence name=" 
        << xmlGetProp(node,(const xmlChar *)"name")
        << " size=" << xmlGetProp(node,(const xmlChar *)"size") << endl;
#endif
      strs.reserve(atoi((char *)xmlGetProp(node,(const xmlChar *)"size")));
      xmlLoadLoopElement(node->children,strs);
      instance->xmlPersistLoadSaveMembers((char *)xmlGetProp(node,(const xmlChar *)"name"),
        strs,
        NULL);      
    }
    // == PTR Sequence ==
    else if (!xmlStrcmp((const xmlChar *)"ptrsequence",node->name))
    {
      vector<string> ids;
      vector<bool>   idstype;

#ifdef PERSIST_DEBUG
      cerr << "    PTR Sequence name=" 
        << xmlGetProp(node,(const xmlChar *)"name")
        << " size=" << xmlGetProp(node,(const xmlChar *)"size") << endl;
#endif
      int n=atoi((char *)xmlGetProp(node,(const xmlChar *)"size"));
      ids.reserve(n);
      idstype.reserve(n);
      xmlLoadLoopPTRElement(node->children,ids,idstype);
      list<CPersistFactory::CPtrStock *> ptrs=
        instance->xmlPersistLoadSaveMembers((char *)xmlGetProp(node,(const xmlChar *)"name"),
                                            ids,
                                            NULL);
      if (ptrs.size() != ids.size())
        throw CException("PERSIST::xmlLoadLoop - some ptr %s are not correctly defined in sequence %s !",
                         (char *)xmlGetProp(node,(const xmlChar *)"name"));
      int i=0;
      for (list<CPersistFactory::CPtrStock *>::iterator P=ptrs.begin();P!=ptrs.end();P++)
      {
        if (idstype[i])
          (*P)->set(m_RegistredData[atoi(ids[i].c_str())]);
        else
        {
          int id=atoi(ids[i].c_str());
          if (id >= 0)
            id+=m_iIdsOffset;
          childFromId(id,(*P));
        }
        i++;
      }
    }
    // == custom ==
    else if (!xmlStrcmp((const xmlChar *)"custom",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << "    Custom =" 
        << xmlNodeGetContent(node) << endl;
#endif
      istringstream in((char *)xmlNodeGetContent(node));
      instance->load(in);
    }
    // ====
    else if (!xmlStrcmp((const xmlChar *)"text",node->name))
    {
      //cerr << "Text = " << xmlNodeGetContent(node) << endl;
    }
    else
      throw CException("PERSIST::xmlLoadLoop - Unknown XML node (%s) !",node->name);
    // next node
    node=node->next;
  }
}
// --------------------------------------------------------
void CPersistFactory::xmlLoadLoopElement(xmlNodePtr node,vector<string>& strs)
{
  while (node != NULL)
  {
    // == Element ==
    if (!xmlStrcmp((const xmlChar *)"element",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << "    Element = " << xmlNodeGetContent(node) << endl; 
#endif
      strs.push_back((char *)xmlNodeGetContent(node));
    }
    else if (!xmlStrcmp((const xmlChar *)"text",node->name))
    {
      //cerr << "Text = " << xmlNodeGetContent(node) << endl;
    }
    else
      throw CException("PERSIST::xmlLoadLoopElement - Unknown XML node (%s) !",node->name);
    // next node
    node=node->next;
  }
}
// --------------------------------------------------------
void CPersistFactory::xmlLoadLoopPTRElement(xmlNodePtr node,vector<string>& ids,vector<bool>& idstype)
{
  while (node != NULL)
  {
    // == PTR Element ==
    if (!xmlStrcmp((const xmlChar *)"ptrelement",node->name))
    {
#ifdef PERSIST_DEBUG
      cerr << "    PTR Element "
        << " id=" << xmlGetProp(node,(const xmlChar *)"id")
        << " mem=" << xmlGetProp(node,(const xmlChar *)"mem") << endl;
#endif
      ids.push_back((char *)xmlGetProp(node,(const xmlChar *)"id"));
      if (!xmlStrcmp((xmlChar *)"data",xmlGetProp(node,(const xmlChar *)"mem")))
        idstype.push_back(true);
      else
        idstype.push_back(false);
    }
    else if (!xmlStrcmp((const xmlChar *)"text",node->name))
    {
      //cerr << "Text = " << xmlNodeGetContent(node) << endl;
    }
    else
      throw CException("PERSIST::xmlLoadLoopPTRElement - Unknown XML node (%s) !",node->name);
    // next node
    node=node->next;
  }
}
// --------------------------------------------------------
#endif
// --------------------------------------------------------
