// ----------------------------------------------------
#include "CWorld.h"
#include "CVertex.h"
#include "UDistance.h"
#include "messages_CWorld.h"
#include <GL/glu.h>
#include <GL/glut.h>
#include <algorithm>
#include <iostream>
#include <time.h>
// ----------------------------------------------------
#ifdef WIN32
#  include <windows.h>
#  define M_PI 3.141592653589793
#endif
// ----------------------------------------------------
#include "CInterface.h"
// ----------------------------------------------------
#include <qcombobox.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#ifdef QT3_SUPPORT
	#include <q3button.h>
#else
	#include <qbutton.h>
#endif

// ----------------------------------------------------
#include <qstatusbar.h>
void display_error(const string msg, int mob, int ligne, int col) {
  ostringstream txt;
  if (mob)   txt << "mobile "<< mob << " ";
  if (ligne) txt << "ligne " << ligne;
  if (col)   txt << ", car " << col;
  if (mob||ligne||col) txt << ":  ";
  txt << msg;
  g_MainWindow->statusBar()->message(txt.str().data());
  g_MainWindow->updateDisplay();
}
void display_warning(const string msg, int mob, int ligne, int col) {
#if 1
  ostringstream txt;
  if (mob)   txt << "mobile "<< mob << " ";
  if (ligne) txt << "ligne " << ligne;
  if (col)   txt << ", car " << col;
  if (mob||ligne||col) txt << ":  ";
  txt << msg;
  g_MainWindow->statusBar()->message(txt.str().data());
#else
  CException(msg);
  // cerr << msg <<"\n\n";
#endif
}
void display_error_clear() {
  g_MainWindow->statusBar()->clear();
}

// ----------------------------------------------------
using namespace std;
// ----------------------------------------------------
CWorld::CWorld() : INetworkGame()
{
  t = -1;
  dt = -1;
  m_tDispError = -1;
}
// ----------------------------------------------------
CWorld::~CWorld()
{
  // delete all mobiles
  for (it_mobile M=m_Mobiles.begin();M!=m_Mobiles.end();M++)
    delete (*M);
  m_Mobiles.clear();
  m_Locals.clear();
  m_Distants.clear();
}
// ----------------------------------------------------
CMobile *CWorld::addMobile()
{
  static CVertex color(rand(),rand(),rand());
  
  CMobile *m=new CMobile();
  // default expression
  m->setExprs("");
  m->compile();
  // color
  unsigned int c=0;
  c+=(((int)color.x()) % 256) << 24;
  c+=(((int)color.y()) % 256) << 16;
  c+=(((int)color.z()) % 256) <<  8;
  m->setColor(c);
  // add
  addMobile(m);
  return (m);
}
// ----------------------------------------------------
void CWorld::addMobile(CMobile *m)
{
  // add to local
  addMobileToLocal(m);
  // register on network
  registerGhostMaster(m);
  // add to mobile list
  m_Mobiles.push_back(m);
}
// ----------------------------------------------------
void CWorld::addMobileToLocal(CMobile *m)
{
  // register to locals
  if (m->getRank() == -1)
  {
    unsigned int i;
    for (i=0;i<m_Locals.size();i++)
      if (m_Locals[i] == NULL)
        break;
    if (i < m_Locals.size())
      m_Locals[i]=m;
    else
      m_Locals.push_back(m);
    m->setRank(i);
    //cerr << "Mobile added rank = " << m->getRank() << endl;
  }
  else
  {
    if (m->getRank() >= m_Locals.size())
      m_Locals.resize(m->getRank()+1);
    if (m_Locals[m->getRank()] != NULL)
      throw CException("[FATAL] CWorld::addMobileToLocal - two mobiles have the same rank !");
    m_Locals[m->getRank()]=m;
  }
  // rebuild mobile combo box
  g_MainWindow->m_cbMobiles->clear();
  for (unsigned int i=0;i<m_Locals.size();i++)
  {
    if (m_Locals[i]==NULL)
      continue;
    g_MainWindow->m_cbMobiles->insertItem(QString("local ")+
					QString("%1").arg(m_Locals[i]->getRank()+1)+
					QString(" (%1@%2)").arg(m_Locals[i]->getRank()+1).arg(NETWORK.clientId()+1));
  }
}
// ----------------------------------------------------
void  CWorld::removeMobile(CMobile *m)
{
  //  cerr << "Suppr. mobile " << m->getMasterClientId() << ' ' << m_Mobiles.size() << " mobiles in pool";
  m_Mobiles.remove(m); 
  //  cerr << " " << m_Mobiles.size() << " remains" << endl;
  if (!m->isMaster())
  {
    for (unsigned int i=0;i<m_Distants[m->getMasterClientId()].size();i++)
    {
      if (m_Distants[m->getMasterClientId()][i] == m)
      {
        m_Distants[m->getMasterClientId()][i]=NULL;
        break;
      }
    }
  }
  else
  {
    for (unsigned int i=0;i<m_Locals.size();i++)
    {
      if (m_Locals[i] == m)
      {
        m_Locals[i]=NULL; // delete (m_Local[i]) ?
        break;
      }
    }
  }
  delete (m);
} 
// ----------------------------------------------------
CMobile *CWorld::getNearest(CMobile *M, int p)
{
  double   dist,        lstdist[5];
  CMobile * ret = NULL,  *(lst)[5];
  int i,j, z=0;

  for (it_mobile C = m_Mobiles.begin();C!=m_Mobiles.end();C++)
    {
      if ((*C) == NULL)
	continue;
      if ((*C) == M)
	continue;
      if (!(*C)->isMaster())
	{
	  if (!(*C)->isExported())
	    continue;
	  if (!g_MainWindow->isClientVisible((*C)->getMasterClientId()))
	    continue;
	}

      dist = (*C)->getPos().dist(M->getPos());

      for (j = 0; j<z; j++)
	if (dist < lstdist[j]) {
	  if (z<p) z++;
	  for (i=z-1; i>j; i--) {
	    lst[i]     = lst[i-1];
	    lstdist[i] = lstdist[i-1];
	  }
	  lst[j]     = (*C);
	  lstdist[j] = dist;
	  break;
	}
      if ((j==z)&&(z<p)) {	  
	lst[z]     = (*C);
	lstdist[z] = dist;
	z++;
      }
    }
  if (p<=z) ret=lst[p-1]; else ret=NULL;
  return ret;
}
// ----------------------------------------------------
CMobile *CWorld::getNearestDir(CMobile *M, double dx, double dy, double a, int p)
{
  double   dist,       lstdist[5];
  CMobile * ret = NULL, *(lst)[5];
  int i,j, z=0;
  CVertex V, dir(dx,dy);
  dir.normalize();
  a = cos(a/2);

  for (it_mobile C = m_Mobiles.begin();C!=m_Mobiles.end();C++)
    {
      if ((*C) == NULL)
	continue;
      if ((*C) == M)
	continue;
      if (!(*C)->isMaster())
	{
	  if (!(*C)->isExported())
	    continue;
	  if (!g_MainWindow->isClientVisible((*C)->getMasterClientId()))
	    continue;
	}

      V = (*C)->getPos()-M->getPos();
      if (dir.dot(V) < a*V.length()) continue;

      dist = (*C)->getPos().dist(M->getPos());

      for (j = 0; j<z; j++)
	if (dist < lstdist[j]) {
	  if (z<p) z++;
	  for (i=z-1; i>j; i--) {
	    lst[i]     = lst[i-1];
	    lstdist[i] = lstdist[i-1];
	  }
	  lst[j]     = (*C);
	  lstdist[j] = dist;
	  break;
	}
      if ((j==z)&&(z<p)) {	  
	lst[z]     = (*C);
	lstdist[z] = dist;
	z++;
      }
    }

  if (p<=z) ret=lst[p-1]; else ret=NULL;
  return ret;
}
// ----------------------------------------------------
void CWorld::step()
{
  for (std::vector<CMobile *>::iterator M=m_Locals.begin();M!=m_Locals.end();M++)
  {
    if ((*M) == NULL)
      continue;
    // ask for network update
    (*M)->networkUpdate();
    // update position
    if ((*M)->isMaster())
      (*M)->update();
    bool contact=false;
    // test for collision with other mobiles
    for (it_mobile C=m_Mobiles.begin();C!=m_Mobiles.end();C++)
    {
      if ((*C) == NULL)
	continue;
      if ((*C) == (*M))
	continue;
      if (!(*C)->isMaster())
      {
	if (!(*C)->isExported())
	  continue;
	if (!g_MainWindow->isClientVisible((*C)->getMasterClientId()))
	  continue;
      }
      if (!(*C)->isVisible())
	continue;
      double  agl;
      CVertex d,sp0,sp1;

      agl=M_PI/2.0+((double)(*M)->getAngle());
      if ((*M)->getIcon() >= 0)
	agl+=CIconFactory::getIcon(((*M)->getIcon()))->get_angle0();
      d=CVertex(cos(agl),sin(agl),0.0);
      sp0=(*M)->getPos()-(*M)->getLength()/2.0*d;
      sp1=(*M)->getPos()+(*M)->getLength()/2.0*d;

      UDistance::segment sM(sp0,sp1);

      agl=M_PI/2.0+((double)(*C)->getAngle());
      if ((*C)->getIcon() >= 0)
	agl+=CIconFactory::getIcon(((*C)->getIcon()))->get_angle0();
      d=CVertex(cos(agl),sin(agl),0.0);
      sp0=(*C)->getPos()-(*C)->getLength()/2.0*d;
      sp1=(*C)->getPos()+(*C)->getLength()/2.0*d;

      UDistance::segment sC(sp0,sp1);
      double r=(*C)->getRadius()+(*M)->getRadius();
      UDistance::hit dist2=UDistance::dist2dsqr(sM,sC);
      if (dist2.first < (r*r))
      {
	contact=true;
	(*M)->collide((*C),dist2.second);
      }
    }
    (*M)->setContact(contact);
    // test for collision with borders
    for (int b=0;b<4;b++)
    {
      double  agl;
      CVertex d,sp0,sp1;
      
      switch (b)
      {
      case WORLD_BORDER_BOTTOM:
	sp0=CVertex(-2.0*WORLD_SCALE,-2.0*WORLD_SCALE);
	sp1=CVertex( 2.0*WORLD_SCALE,-2.0*WORLD_SCALE);
	break;
      case WORLD_BORDER_RIGHT:
	sp0=CVertex(2.0*WORLD_SCALE,-2.0*WORLD_SCALE);
	sp1=CVertex(2.0*WORLD_SCALE, 2.0*WORLD_SCALE);
	break;
      case WORLD_BORDER_TOP:
	sp0=CVertex( 2.0*WORLD_SCALE,2.0*WORLD_SCALE);
	sp1=CVertex(-2.0*WORLD_SCALE,2.0*WORLD_SCALE);
	break;
      case WORLD_BORDER_LEFT:
	sp0=CVertex(-2.0*WORLD_SCALE, 2.0*WORLD_SCALE);
	sp1=CVertex(-2.0*WORLD_SCALE,-2.0*WORLD_SCALE);
	break;
      }
	
      UDistance::segment sB(sp0,sp1);
      
      agl=M_PI/2.0+((double)(*M)->getAngle());
      if ((*M)->getIcon() >= 0)
	agl+=CIconFactory::getIcon(((*M)->getIcon()))->get_angle0();
      d=CVertex(cos(agl),sin(agl),0.0);
      sp0=(*M)->getPos()-(*M)->getLength()/2.0*d;
      sp1=(*M)->getPos()+(*M)->getLength()/2.0*d;
      
      UDistance::segment sM(sp0,sp1);
      double r=(*M)->getRadius()+WORLD_BORDER_RADIUS;
      UDistance::hit dist2=UDistance::dist2dsqr(sM,sB);
      if (dist2.first < (r*r))
      {
	(*M)->collideBorder(b);
      }
    }
    
  }
  if (NETWORK.isEnabled())
    processGhosts();

  setClic(0);
  setUnClic(0);
}
// ----------------------------------------------------
CWorld& CWorld::getUniqueInstance()
{
  static CWorld world;
  
  return (world);
}
// ----------------------------------------------------
IGhost *CWorld::createGhost(int t,int id,const unsigned char *d,int l)
{
  //std::cerr << "CWorld::createGhost - adding ghost ..." << endl;
  CMobile *m=new CMobile(t,id,d,l);
  m_Mobiles.push_back(m);
  return (m);
}
// ----------------------------------------------------
void CWorld::onNewGhost(IGhost *g)
{
  // add to distant client list
  if (m_Distants.size() <= g->getMasterClientId())
    m_Distants.resize(g->getMasterClientId()+1);
  m_Distants[g->getMasterClientId()].push_back(static_cast<CMobile *>(g));
  sort(m_Distants[g->getMasterClientId()].begin(),
       m_Distants[g->getMasterClientId()].end(),
       CMobile::less());
}
// ----------------------------------------------------
void CWorld::sampleTime()
{
  double nt = (double)SDL_GetTicks()/1000.0;
  if (t>0) {
    double ndt = nt-t;
    if (dt<0) dt = ndt;
    else   dt = .1*ndt+.9*dt;  // lissage de dt
  }
  t = nt;
}
/*
double CWorld::getTime()
{
  double t = ((double)SDL_GetTicks()/1000.0);
  return t;
}

double CWorld::getTimeInterval() {
  static double t = -1, dt_=0;
  double nt = getTime();
  double dt = (t<0) ? 0 : nt-t;
  dt_ = dt = .1*dt+.9*dt_;  // lissage de dt  
  t=nt;
  return dt;
}
void CWorld::resetTime()
{ t = dt = -1;
} 
*/
// ----------------------------------------------------
void CWorld::drawMobiles()
{
  // draw mobiles
  glPushMatrix();
  glScaled(1.0/WORLD_SCALE,1.0/WORLD_SCALE,1.0/WORLD_SCALE);
  glColor3d(1.0,0.0,0.0);
  for (it_mobile I=m_Mobiles.begin();
       I!=m_Mobiles.end();I++)
  {
    if ((*I)->isMaster() || (g_MainWindow->isClientVisible((*I)->getMasterClientId()) 
			     && !(*I)->isMaster() 
			     && (*I)->isExported()))
      (*I)->draw();
  }
  glPopMatrix();
}
// ----------------------------------------------------
void CWorld::drawString(const char *text)
{
  int len = (int) strlen(text);

  glPushMatrix();
  for (unsigned int i=0;i<len;i++)
    //glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_10, *p);
    glutStrokeCharacter(GLUT_STROKE_ROMAN, text[i]);
  glPopMatrix();
}
// ----------------------------------------------------
void CWorld::drawMobileString(CMobile *m,const char *str)
{
  static double mv[16],prj[16];
  static int    v[4];
  GLdouble      x,y,z;

  glGetDoublev(GL_MODELVIEW_MATRIX,mv);
  glGetDoublev(GL_PROJECTION_MATRIX,prj);
  glGetIntegerv(GL_VIEWPORT,v);

  CVertex p=m->pos()/WORLD_SCALE;
  gluProject(p.x(),p.y(),p.z(),
	     mv,prj,v,
	     &x,&y,&z);

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  gluOrtho2D(0.0,1.0,0.0,1.0);

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();

  //glRasterPos2d(x/(double)v[2],y/(double)v[3]);
  glTranslated(x/(double)v[2],y/(double)v[3],0.0);
  glScaled(0.2/v[2],0.2/v[3],0.0);

  glColor3d(0.0,0.0,0.0);
  glLineWidth(4.0);
  drawString(str);

  glColor3d(((m->getColor() & 0xFF000000) >> 24)/255.0,
	    ((m->getColor() & 0x00FF0000) >> 16)/255.0,
	    ((m->getColor() & 0x0000FF00) >>  8)/255.0);
  glLineWidth(2.0);
  drawString(str);
  glLineWidth(1.0); // $FAB

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
}
// ----------------------------------------------------
void CWorld::drawNames()
{
  static char str[64];

  glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  for (unsigned int i=0;i<m_Locals.size();i++)
  { 
    if (m_Locals[i] == NULL)
      continue;
    sprintf(str,"local %d",i+1);
    if (m_Locals[i]->isMaster() || g_MainWindow->isClientVisible(m_Locals[i]->getMasterClientId()))
      drawMobileString(m_Locals[i],str);
  }
  for (unsigned int i=0;i<m_Distants.size();i++)
  {
    for (unsigned int j=0;j<m_Distants[i].size();j++)
    {
      if (m_Distants[i][j] == NULL)
	continue;
      if (!m_Distants[i][j]->isExported())
	continue;
      sprintf(str,"%d@%d",j+1,i+1);
      if (m_Distants[i][j]->isMaster() || g_MainWindow->isClientVisible(m_Distants[i][j]->getMasterClientId()))
	drawMobileString(m_Distants[i][j],str);
    }
  }
  glPopAttrib();
}
// ----------------------------------------------------
bool CWorld::drawState()
{
  if (!NETWORK.error())
    return false;

  if(m_tDispError == -1.0)
	  m_tDispError = clock();
  else 
	  if ( (clock()-m_tDispError)/(CLOCKS_PER_SEC) > 3.0)
		  return true;

  glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  gluOrtho2D(0.0,1.0,0.0,1.0);

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();

  glTranslated(0.25,0.5,0.0);
  glScaled(0.0003,0.0003,1.0);
  glColor3d(1.0,0.0,0.0);
  glLineWidth(3.0);
  drawString("Disconnected from server !!");
  glLineWidth(1.0);

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();

  glPopAttrib();

  return true;
}
// ----------------------------------------------------
void CWorld::drawFrame()
{
  static char str[64];

  glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);

  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glLineWidth(2.0);

  glColor3d(0.0,0.0,0.0);
  glBegin(GL_LINES);
  glVertex2d(0.0,-1.0);
  glVertex2d(0.0,1.0);

  glVertex2d(-1.0,0.0);
  glVertex2d(1.0,0.0);
  glEnd();

  glLineWidth(1.0);

  sprintf(str,"100");
  glPushMatrix();
  glTranslated(0.9,0.01,0.0);
  glScaled(0.0003,0.0003,1.0);
  drawString(str);
  glPopMatrix();  

  sprintf(str,"-100");
  glPushMatrix();
  glTranslated(-0.9,0.01,0.0);
  glScaled(0.0003,0.0003,1.0);
  drawString(str);
  glPopMatrix();  

  sprintf(str,"-100");
  glPushMatrix();
  glTranslated(0.01,-0.9,0.0);
  glScaled(0.0003,0.0003,1.0);
  drawString(str);
  glPopMatrix();  

  sprintf(str,"100");
  glPushMatrix();
  glTranslated(0.01,0.9,0.0);
  glScaled(0.0003,0.0003,1.0);
  drawString(str);
  glPopMatrix();  

  sprintf(str,"(0,0)");
  glPushMatrix();
  glTranslated(0.01,0.01,0.0);
  glScaled(0.0003,0.0003,1.0);
  drawString(str);
  glPopMatrix();  
  
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();  
  glPopAttrib();

  glLineWidth(1.0);
}
// ----------------------------------------------------
CMobile *CWorld::getLocalMobile(int n)
{
  if ((n < 0) || (n >= m_Locals.size()))
    throw CException(MSG_access_loc,n+1,m_Locals.size());
  if (m_Locals[n] == NULL)
    throw CException(MSG_access_loc_del,n+1);
  return (m_Locals[n]);
}
// ----------------------------------------------------
CMobile *CWorld::getDistantMobile(int m,int c)
{
  if (c == NETWORK.clientId())
    return (getLocalMobile(m));
  if (c < 0 || c >= m_Distants.size())
    throw CException(MSG_access_client,c+1,m_Distants.size());
  if (m < 0 || m >= m_Distants[c].size())
    throw CException(MSG_access_dist,c+1,m+1,m_Distants[c].size());
  if (m_Distants[c][m] == NULL)
    throw CException(MSG_access_dist_del,c+1,m+1);
  return (m_Distants[c][m]);
}
// ----------------------------------------------------
int CWorld::getMobileLocalId(CMobile* m)
{
  int n = m_Locals.size(), i = 0;
  for (i=0;i<n;i++)
  {
    if (m_Locals[i] == m)
      return (1+i);
  }
  throw CException(MSG_nomob);
}
// ----------------------------------------------------
CMobile *CWorld::getNextMobile(CMobile*m)
{
  int N = m_Locals.size(), i = 0;
  CMobile *ret = NULL;

  if( N == 0 )
    return NULL;

  while( m_Locals[i] != m )
    i++;
  int mob = ( i + 1 ) % N;
  i = 0;
  while( i < N + 1 )
    {
      if( m_Locals[mob] != NULL )
	{
	  ret = m_Locals[mob];
	  i = N+1;
	}
      else
	{
	  i++;
	  mob = ( mob + 1 ) % N;
	}
    }
  return m_Locals[mob];
}
// ----------------------------------------------------
CMobile *CWorld::getPrevMobile(CMobile*m)
{
  int N = m_Locals.size(), i = 0;
  CMobile *ret = NULL;
  if( N == 0 ) return NULL;
  while( m_Locals[i] != m )
    i++;
  int mob = ( i + N - 1 ) % N;
  i = 0;
  while( i < N + 1 )
    {
      if( m_Locals[mob] != NULL )
	{
	  ret = m_Locals[mob];
	  i = N+1;
	}
      else
	{
	  i++;
	  mob = ( mob + N - 1 ) % N;
	}
    }
  return m_Locals[mob];
}
// ----------------------------------------------------
CMobile *CWorld::getMobile(int m,int c)
{
  if (c == -1)
    return (getLocalMobile(m));
  else
    return (getDistantMobile(m,c));
}
// ----------------------------------------------------
void CWorld::onClientConnection(int cli)
{
  // call parent !
  INetworkGame::onClientConnection(cli);
  // add button
  g_MainWindow->addVisuButton(cli);
}
// ----------------------------------------------------
void CWorld::onAllGhostsReceived(int cli)
{
  // call parent !
  INetworkGame::onAllGhostsReceived(cli);
  // update all master export / visible
  for (unsigned int i=0;i<m_Locals.size();i++)
  {
    if (m_Locals[i] == NULL)
      continue;
    m_Locals[i]->updateExport();
    m_Locals[i]->updateVisible();
  }
}
// ----------------------------------------------------
void CWorld::onClientDisconnection(int cli)
{
  // call parent !
  INetworkGame::onClientDisconnection(cli);
  // remove button
  g_MainWindow->removeVisuButton(cli);
  // reset client
  resetClient(cli);
}
// ----------------------------------------------------
void CWorld::onNetworkReady()
{
  // call parent !
  INetworkGame::onNetworkReady();
  // create disabled button
  QPushButton *btn;
  int cli=NETWORK.clientId();
  g_MainWindow->addVisuButton(cli,false);
}
// ----------------------------------------------------
CMobile *CWorld::getMouse() 
{
  static CMobile *mouse=NULL;
  if (mouse == NULL)
  {
    mouse=new CMobile();
    mouse->setGhostMasterId(0); // allows to avoid to distinguish mouse from 
                                // other mobiles. (mouse is not registred
                                // on the network !)
    PERSIST.unregister(mouse);
  }
  return (mouse);
}
// ----------------------------------------------------
CMobile *CWorld::getCamera() 
{
  static CMobile *camera=NULL;
  if (camera == NULL)
  {
    camera=new CMobile();
    camera->setGhostMasterId(0);  // allows to avoid to distinguish camera from 
                                  // other mobiles. (camera is not registred
                                  // on the network)
    camera->setVisible(0); // pas de trace
    camera->setColor(0xffffff00);
    camera->setVar(2,1.); // pointe vers z
    PERSIST.unregister(camera);
  }
  return (camera);
}
// ----------------------------------------------------
CMobile *CWorld::getLight1() 
{
  static CMobile *light=NULL;
  if (light == NULL)
  {
    light=new CMobile();
    light->setGhostMasterId(0);  // allows to avoid to distinguish light from 
                                 // other mobiles. (light is not registred
                                 // on the network)
    light->setColor(0xffffff00);
    light->setHeight(1e8);
    PERSIST.unregister(light);
  }
  return (light);
}
// ----------------------------------------------------
CMobile *CWorld::getLight2() 
{
  static CMobile *light=NULL;
  if (light == NULL)
  {
    light=new CMobile();
    light->setGhostMasterId(0);  // allows to avoid to distinguish light from 
                                 // other mobiles. (light is not registred
                                 // on the network)
    light->setColor(0x00000000);
    light->setHeight(1e8);
    PERSIST.unregister(light);
  }
  return (light);
}
// ----------------------------------------------------
CMobile *CWorld::getLight3() 
{
  static CMobile *light=NULL;
  if (light == NULL)
  {
    light=new CMobile();
    light->setGhostMasterId(0);  // allows to avoid to distinguish light from 
                                 // other mobiles. (light is not registred
                                 // on the network)
    light->setColor(0x00000000);
    light->setHeight(1e8);
    PERSIST.unregister(light);
  }
  return (light);
}
// ----------------------------------------------------
void  CWorld::reset()
{
  static network_msg msg;

  // clear locals
  m_Locals.clear();
  // delete all local mobiles
  it_mobile M=m_Mobiles.begin();
  while (M != m_Mobiles.end())
  {
    CMobile *m=(*M);
    if (m->isMaster())
    {
      M=m_Mobiles.erase(M);
      delete (m);
    }
    else
      M++;
  }
  // send message to others
  NETWORK.clientMessageFromData(WORLD_MSG_RESET,NULL,0,&msg);
  NETWORK.clientSend(NTWRK_SECURE,NTWRK_TO_OTHERS,&msg);  
}
// ----------------------------------------------------
void  CWorld::resetClient(int cli)
{
  // forget distant ghosts
  if (cli < m_Distants.size())
    m_Distants[cli].clear();
}
// ----------------------------------------------------
void  CWorld::processClientMessage(const network_msg& m)
{
  // call parent !
  INetworkGame::processClientMessage(m);
  // get message
  switch (m.m_Type)
  {
  case WORLD_MSG_RESET:
    {
      //cerr << "WORLD_MSG_RESET from " << (int)m.m_ClientId << endl;
      // reset client data
      resetClient((int)m.m_ClientId);
      break;
    }
  }
}
// ----------------------------------------------------
