#include "interface.h"

#include <qvariant.h>
#include "CViewer.h"
/*
 *  Constructs a Interface as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 */
Interface::Interface(QWidget* parent, const char* name, Qt::WFlags fl)
    : Q3MainWindow(parent, name, fl)
{
    setupUi(this);

    (void)statusBar();

    // signals and slots connections
    connect(m_cbBordRPresets, SIGNAL(activated(int)), this, SLOT(loadBordRPreset(int)));
    connect(m_cbBordTPresets, SIGNAL(activated(int)), this, SLOT(loadBordTPreset(int)));
    connect(m_cbBordBPresets, SIGNAL(activated(int)), this, SLOT(loadBordBPreset(int)));
    connect(m_cbBordLPresets, SIGNAL(activated(int)), this, SLOT(loadBordLPreset(int)));
    connect(PushButton17_2, SIGNAL(released()), this, SLOT(selectPrev()));
    connect(PushButton17, SIGNAL(released()), this, SLOT(selectNext()));
    connect(m_cbMobiles, SIGNAL(activated(int)), this, SLOT(activateMobile(int)));
    connect(PushButton25_4, SIGNAL(released()), this, SLOT(resetSelected()));
    connect(m_btnStop, SIGNAL(released()), this, SLOT(startStopSelected()));
    connect(m_leX, SIGNAL(returnPressed()), this, SLOT(setValues()));
    connect(m_leDx, SIGNAL(returnPressed()), this, SLOT(setValues()));
    connect(m_leWidth, SIGNAL(returnPressed()), this, SLOT(setValues()));
    connect(m_leY, SIGNAL(returnPressed()), this, SLOT(setValues()));
    connect(m_leDy, SIGNAL(returnPressed()), this, SLOT(setValues()));
    connect(m_leHeight, SIGNAL(returnPressed()), this, SLOT(setValues()));
    connect(m_leAgl, SIGNAL(returnPressed()), this, SLOT(setValues()));
    connect(m_cbVisible, SIGNAL(released()), this, SLOT(visibleChanged()));
    connect(m_cbExport, SIGNAL(released()), this, SLOT(exportChanged()));
    connect(m_cbIcons, SIGNAL(activated(int)), this, SLOT(iconChange(int)));
    connect(m_btnCancel, SIGNAL(released()), this, SLOT(cancelExprs()));
    connect(m_btnApply, SIGNAL(released()), this, SLOT(applyAll()));
    connect(m_cbMvtPresets, SIGNAL(activated(int)), this, SLOT(loadMvtPreset(int)));
    connect(m_cbStartPresets, SIGNAL(activated(int)), this, SLOT(loadStartPreset(int)));
    connect(m_cbCollidePresets, SIGNAL(activated(int)), this, SLOT(loadCollidePreset(int)));
    connect(PushButton3, SIGNAL(released()), this, SLOT(addMobile()));
    connect(PushButton24, SIGNAL(released()), this, SLOT(supprSelected()));
    connect(PushButton3_2, SIGNAL(released()), this, SLOT(cloneSelected()));
    connect(m_btnSaveMobile, SIGNAL(released()), this, SLOT(saveSelected()));
    connect(PushButton25_2, SIGNAL(released()), this, SLOT(load()));
    connect(PushButton13, SIGNAL(released()), this, SLOT(exportAll()));
    connect(PushButton13_2, SIGNAL(released()), this, SLOT(exportNone()));
    connect(PushButton12, SIGNAL(released()), this, SLOT(viewAll()));
    connect(PushButton12_2, SIGNAL(released()), this, SLOT(viewNone()));
    connect(PushButton25_2_2, SIGNAL(released()), this, SLOT(loadSession()));
    connect(m_btnSaveSession, SIGNAL(released()), this, SLOT(saveSession()));
    connect(PushButton16, SIGNAL(released()), this, SLOT(resetSession()));
    connect(m_cbColors, SIGNAL(activated(int)), this, SLOT(changeColor(int)));
    connect(m_teExprs, SIGNAL(textChanged()), this, SLOT(exprsChanged()));
    connect(m_teExprsBL, SIGNAL(textChanged()), this, SLOT(exprsChanged()));
    connect(m_teExprsBR, SIGNAL(textChanged()), this, SLOT(exprsChanged()));
    connect(m_teExprsBT, SIGNAL(textChanged()), this, SLOT(exprsChanged()));
    connect(m_teExprsBB, SIGNAL(textChanged()), this, SLOT(exprsChanged()));
    connect(m_teExprsCollide, SIGNAL(textChanged()), this, SLOT(exprsChanged()));
    connect(m_teExprsStart, SIGNAL(textChanged()), this, SLOT(exprsChanged()));
}

/*
 *  Destroys the object and frees any allocated resources
 */
Interface::~Interface()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void Interface::languageChange()
{
    retranslateUi(this);
}

void Interface::activateMobile(int)
{
    qWarning("Interface::activateMobile(int): Not implemented yet");
}

void Interface::addMobile()
{
    qWarning("Interface::addMobile(): Not implemented yet");
}

void Interface::applyExprs()
{
    qWarning("Interface::applyExprs(): Not implemented yet");
}

void Interface::cancelExprs()
{
    qWarning("Interface::cancelExprs(): Not implemented yet");
}

void Interface::changeColor(int)
{
    qWarning("Interface::changeColor(int): Not implemented yet");
}

void Interface::cloneSelected()
{
    qWarning("Interface::cloneSelected(): Not implemented yet");
}

void Interface::exportAll()
{
    qWarning("Interface::exportAll(): Not implemented yet");
}

void Interface::exportChanged()
{
    qWarning("Interface::exportChanged(): Not implemented yet");
}

void Interface::exportNone()
{
    qWarning("Interface::exportNone(): Not implemented yet");
}

void Interface::exprsChanged()
{
    qWarning("Interface::exprsChanged(): Not implemented yet");
}

void Interface::iconChange(int)
{
    qWarning("Interface::iconChange(int): Not implemented yet");
}

void Interface::load()
{
    qWarning("Interface::load(): Not implemented yet");
}

void Interface::loadBordBPreset(int)
{
    qWarning("Interface::loadBordBPreset(int): Not implemented yet");
}

void Interface::loadBordLPreset(int)
{
    qWarning("Interface::loadBordLPreset(int): Not implemented yet");
}

void Interface::loadBordRPreset(int)
{
    qWarning("Interface::loadBordRPreset(int): Not implemented yet");
}

void Interface::loadBordTPreset(int)
{
    qWarning("Interface::loadBordTPreset(int): Not implemented yet");
}

void Interface::loadCollidePreset(int)
{
    qWarning("Interface::loadCollidePreset(int): Not implemented yet");
}

void Interface::loadMvtPreset(int)
{
    qWarning("Interface::loadMvtPreset(int): Not implemented yet");
}

void Interface::loadSession()
{
    qWarning("Interface::loadSession(): Not implemented yet");
}

void Interface::loadStartPreset(int)
{
    qWarning("Interface::loadStartPreset(int): Not implemented yet");
}

void Interface::selectPrev()
{
    qWarning("Interface::selectPrev(): Not implemented yet");
}

void Interface::resetSelected()
{
    qWarning("Interface::resetSelected(): Not implemented yet");
}

void Interface::resetSession()
{
    qWarning("Interface::resetSession(): Not implemented yet");
}

void Interface::saveSelected()
{
    qWarning("Interface::saveSelected(): Not implemented yet");
}

void Interface::saveSession()
{
    qWarning("Interface::saveSession(): Not implemented yet");
}

void Interface::selectNext()
{
    qWarning("Interface::selectNext(): Not implemented yet");
}

void Interface::setValues()
{
    qWarning("Interface::setValues(): Not implemented yet");
}

void Interface::startStopSelected()
{
    qWarning("Interface::startStopSelected(): Not implemented yet");
}

void Interface::supprSelected()
{
    qWarning("Interface::supprSelected(): Not implemented yet");
}

void Interface::viewAll()
{
    qWarning("Interface::viewAll(): Not implemented yet");
}

void Interface::viewNone()
{
    qWarning("Interface::viewNone(): Not implemented yet");
}

void Interface::visibleChanged()
{
    qWarning("Interface::visibleChanged(): Not implemented yet");
}

void Interface::applyAll()
{
    qWarning("Interface::applyAll(): Not implemented yet");
}

